#!/usr/bin/perl
use ChunkAlignment;
use WordQuery;
use Lingua::EN::Tagger;
use WordNet::Similarity::vector;
use List::Util qw( min max );
use WordNet::QueryData;

use strict;
use warnings;


#---------------------------------------------
#                     Run 1
#---------------------------------------------

#initialize variables
my $tagger = new Lingua::EN::Tagger;
my $qd = WordNet::QueryData->new;
my $measure=WordNet::Similarity::vector->new($qd);
my $wq = WordQuery->new();
my $maxLevenshtein = 0;
my $removeStopWords = 1;
my $wordToWordSimilarityThreshold = 0.9;

#trainingSets
my $inFileName = 'data/training/STS.input.headlines.txt';
my $outFileName = 'results/training/headlines_withStopRem.txt';
&compareFiles($inFileName, $outFileName);

#testSet
$inFileName = 'data/test/STS2016.input.postediting.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.withStopRem.postediting.txt';
&compareFiles($inFileName, $outFileName);

$inFileName = 'data/test/STS2016.input.answer-answer.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.withStopRem.answer-answer.txt';
&compareFiles($inFileName, $outFileName);

$inFileName = 'data/test/STS2016.input.headlines.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.withStopRem.headlines.txt';
&compareFiles($inFileName, $outFileName);

$inFileName = 'data/test/STS2016.input.plagiarism.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.withStopRem.plagiarism.txt';
&compareFiles($inFileName, $outFileName);

$inFileName = 'data/test/STS2016.input.question-question.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.withStopRem.question-question.txt';
&compareFiles($inFileName, $outFileName);


=comment
#---------------------------------------------
#                     Run 2
#---------------------------------------------


#initialize variables
$tagger = new Lingua::EN::Tagger;
$qd = WordNet::QueryData->new;
$measure=WordNet::Similarity::vector->new($qd);
$wq = WordQuery->new();
$maxLevenshtein = 0;
$removeStopWords = 0;
$wordToWordSimilarityThreshold = 0.9;

#trainingSets
$inFileName = 'data/training/STS.input.headlines.txt';
$outFileName = 'results/training/headlines_noStopRem.txt';
&compareFiles($inFileName, $outFileName);

#testSets
$inFileName = 'data/test/STS2016.input.answer-answer.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.noStopRem.answer-answer.txt';
&compareFiles($inFileName, $outFileName);

$inFileName = 'data/test/STS2016.input.headlines.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.noStopRem.headlines.txt';
&compareFiles($inFileName, $outFileName);

$inFileName = 'data/test/STS2016.input.plagiarism.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.noStopRem.plagiarism.txt';
&compareFiles($inFileName, $outFileName);

$inFileName = 'data/test/STS2016.input.postediting.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.noStopRem.postediting.txt';
&compareFiles($inFileName, $outFileName);

$inFileName = 'data/test/STS2016.input.question-question.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.noStopRem.question-question.txt';
&compareFiles($inFileName, $outFileName);

#---------------------------------------------
#                     Run 3
#---------------------------------------------


#initialize variables
$tagger = new Lingua::EN::Tagger;
$qd = WordNet::QueryData->new;
$measure=WordNet::Similarity::vector->new($qd);
$wq = WordQuery->new();
$maxLevenshtein = 2;
$removeStopWords = 1;
$wordToWordSimilarityThreshold = 0.9;

#trainingSets
$inFileName = 'data/training/STS.input.headlines.txt';
$outFileName = 'results/training/headlines_withLeven.txt';
&compareFiles($inFileName, $outFileName);

#testSets
$inFileName = 'data/test/STS2016.input.answer-answer.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.withLeven.answer-answer.txt';
&compareFiles($inFileName, $outFileName);

$inFileName = 'data/test/STS2016.input.headlines.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.withLeven.headlines.txt';
&compareFiles($inFileName, $outFileName);

$inFileName = 'data/test/STS2016.input.plagiarism.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.withLeven.plagiarism.txt';
&compareFiles($inFileName, $outFileName);

$inFileName = 'data/test/STS2016.input.postediting.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.withLeven.postediting.txt';
&compareFiles($inFileName, $outFileName);

$inFileName = 'data/test/STS2016.input.question-question.ascii';
$outFileName = 'results/test/STS2016.OUTPUT.VRep.withLeven.question-question.txt';
&compareFiles($inFileName, $outFileName);
=cut


#compare the files
sub compareFiles
{
    #open the input file
    my $inFile = shift;
    my $outFile = shift;
    open FILE, $inFile;
    open (OUT, ">>".$outFile);

    #find the similarity score for each line
    my $sentenceNumber = 0;
    while (my $line = <FILE>) {
	$sentenceNumber++;
	print "-------------  $sentenceNumber    ----------------\n";
	print "$line\n";

	#break the line into 2 sentences
	#my @sentences = split /\.\s+/, $line;   #this worked
	#my @sentences = split /((\.\s+)|(\s\s+))/, $line;
	my @sentences = split /\t/, $line;

	print join('-',@sentences)."\n";
	my $sentence1 = $sentences[0];
	my $sentence2 = $sentences[1];
	print "Sentences Split\n";
	print "*  $sentence1\n";
	if (!(defined $sentence2)) {
	    #print "******  QUITTING, sentence2 is undefined\n";
	    print OUT "ERR\n"; 
	    next;
	}

	print "*  $sentence2\n";

	#remove punctuation
	$sentence1 =~ s/[.,;'?!"]//g;
	$sentence2 =~ s/[.,;'?!"]//g;
	print "Punctuation Removed: \n";
	print "   $sentence1\n";
	print "   $sentence2\n";
       
        #get POS Tags for each sentence
	my @posTags1 = &ChunkAlignment::getPOSTags($sentence1, $tagger);
	my @posTags2 = &ChunkAlignment::getPOSTags($sentence2, $tagger);
	print "POS Tags:\n";
	print "   @posTags1\n";
	print "   @posTags2\n";

	#swap if needed
	if (scalar @posTags1 > scalar @posTags2) {
	    my @temp = @posTags1;
	    @posTags1 = @posTags2;
	    @posTags2 = @temp;

	    my $temp = $sentence1;
	    $sentence1 = $sentence2;
	    $sentence2 = $temp;
	}
	
	#split the sentences into tokens
	my @tokens1 = split ' ', $sentence1;
	my @tokens2 = split ' ', $sentence2;
	print "Tokens Split:\n";
	print "   @tokens1\n";
	print "   @tokens2\n";
	
	my $sameLength = "false";
	if (scalar @tokens1 == scalar @posTags1) {
	    $sameLength = "true";
	}
	print "   tokens1 == posTags1 - $sameLength\n";
	if ($sameLength eq "false") {
	    print OUT "ERR\n";
	    next;
	}

	if (scalar @tokens2 == scalar @posTags2) {
	    $sameLength = "true";
	}
	print "   tokens2 == posTags2 - $sameLength\n";
	if ($sameLength eq "false") {
	    print OUT "ERR\n";
	    next;
	}

	#standardize the tokens
	@tokens1 = &ChunkAlignment::standardizeTokens(\@tokens1);
	@tokens2 = &ChunkAlignment::standardizeTokens(\@tokens2);
	print "Chunks Standardized:\n";
	print "   @tokens1\n";
	print "   @tokens2\n";

	#remove stop words
	if ($removeStopWords) {
	    @tokens1 = &ChunkAlignment::removeStopWords(\@tokens1, \@posTags1);
	    @tokens2 = &ChunkAlignment::removeStopWords(\@tokens2, \@posTags2);
	}
	print "Stop Words Removed:\n";
	print "   @tokens1\n";
	print "   @tokens2\n";

	
	#compute word to word similarities
	my @wordNetTerms1 = @tokens1;
	my @wordNetTerms2 = @tokens2;
	my @similarities = @{ 
	    &ChunkAlignment::computeWordToWordSimilarities(\@tokens1, \@tokens2,
							   \@wordNetTerms1, \@wordNetTerms2,
							   $measure, $wq,
							   $wordToWordSimilarityThreshold, 
							   $maxLevenshtein) };

	print "Similarities:\n";
	print "   @similarities\n";

	#find the max similarity for each word
	my @maxSims = ();
	foreach my $scoreRef(@similarities) {
	    my @scores = @{ $scoreRef };
	    push @maxSims, max @scores;
	}
	print "\n@maxSims\n";

	#find the weighted average
	my $numScores = 0;   #TODO I should not include punctuation, maybe just do content words...so a quick if not "" filter
	my $finalScore = 0;
	for (my $i = 0; $i < scalar @maxSims; $i++) {
	    if ($tokens1[$i] ne "") {
		$numScores++;
		$finalScore += $maxSims[$i];
	    }
	}
	#avoid division by 0 (when no content words are present)
	if ($numScores > 0) {
	    $finalScore /= $numScores; #will scale to 0-1
	}
	$finalScore *= 5; #will scale to 0-5

	#output the results
	print "finalScore = $finalScore\n";
	print OUT "$finalScore\n";
    }
    
}
